'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var backendCommon = require('@backstage/backend-common');
var express = require('express');
var Router = require('express-promise-router');
var utilArnParser = require('@aws-sdk/util-arn-parser');
var awsCorePluginForBackstageCommon = require('@aws/aws-core-plugin-for-backstage-common');
var integrationAwsNode = require('@backstage/integration-aws-node');
var clientCodebuild = require('@aws-sdk/client-codebuild');
var awsCodebuildPluginForBackstageCommon = require('@aws/aws-codebuild-plugin-for-backstage-common');
var backendPluginApi = require('@backstage/backend-plugin-api');
var alpha = require('@backstage/plugin-catalog-node/alpha');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var express__default = /*#__PURE__*/_interopDefaultLegacy(express);
var Router__default = /*#__PURE__*/_interopDefaultLegacy(Router);

class DefaultAwsCodeBuildService {
  constructor(logger, catalogApi, resourceLocator, credsManager) {
    this.logger = logger;
    this.catalogApi = catalogApi;
    this.resourceLocator = resourceLocator;
    this.credsManager = credsManager;
  }
  static async fromConfig(config, options) {
    var _a;
    const credsManager = integrationAwsNode.DefaultAwsCredentialsManager.fromConfig(config);
    const resourceLocator = (_a = options == null ? void 0 : options.resourceLocator) != null ? _a : await awsCorePluginForBackstageCommon.AwsResourceLocatorFactory.fromConfig(config, options.logger);
    return new DefaultAwsCodeBuildService(
      options.logger,
      options.catalogApi,
      resourceLocator,
      credsManager
    );
  }
  async getProjectsByEntity(entityRef) {
    var _a;
    (_a = this.logger) == null ? void 0 : _a.debug(`Fetch CodeBuild projects for ${entityRef}`);
    const arns = await this.getCodeBuildArnsForEntity(entityRef);
    const projects = await Promise.all(
      arns.map(async (arn) => {
        var _a2;
        const { region, accountId, resource } = utilArnParser.parse(arn);
        const projectName = resource.split("/")[1];
        const client = await this.getClient(region, arn);
        const projectResponse = await client.send(
          new clientCodebuild.BatchGetProjectsCommand({
            names: [projectName]
          })
        );
        const project = projectResponse.projects[0];
        const buildIds = await client.send(
          new clientCodebuild.ListBuildsForProjectCommand({
            projectName
          })
        );
        let builds = [];
        if (buildIds.ids) {
          const output = await client.send(
            new clientCodebuild.BatchGetBuildsCommand({
              ids: buildIds.ids.slice(0, 5)
            })
          );
          builds = (_a2 = output.builds) != null ? _a2 : [];
        }
        return {
          project,
          projectName,
          projectRegion: region,
          projectAccountId: accountId,
          builds
        };
      })
    );
    return {
      projects
    };
  }
  async getCodeBuildArnsForEntity(entityRef) {
    const entity = await this.catalogApi.getEntityByRef(entityRef);
    if (!entity) {
      throw new Error(`Failed to find entity ${JSON.stringify(entityRef)}`);
    }
    const annotation = awsCorePluginForBackstageCommon.getOneOfEntityAnnotations(entity, [
      awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_ARN_ANNOTATION,
      awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_TAGS_ANNOTATION
    ]);
    if (!annotation) {
      throw new Error("Annotation not found on entity");
    }
    let arns;
    if (annotation.name === awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_TAGS_ANNOTATION) {
      arns = await this.resourceLocator.getResourceArns({
        resourceType: "AWS::CodeBuild::Project",
        tagString: annotation.value
      });
    } else {
      arns = [annotation.value];
    }
    return Promise.resolve(arns);
  }
  async getClient(region, arn) {
    const credentialProvider = await this.credsManager.getCredentialProvider({
      arn
    });
    return new clientCodebuild.CodeBuildClient({
      region,
      customUserAgent: "aws-codebuild-plugin-for-backstage",
      credentialDefaultProvider: () => credentialProvider.sdkCredentialProvider
    });
  }
}

async function createRouter(options) {
  const { logger, awsCodeBuildApi } = options;
  const router = Router__default["default"]();
  router.use(express__default["default"].json());
  router.get(
    "/v1/entity/:namespace/:kind/:name/projects",
    async (request, response) => {
      const { namespace, kind, name } = request.params;
      const services = await awsCodeBuildApi.getProjectsByEntity({
        kind,
        namespace,
        name
      });
      response.status(200).json(services);
    }
  );
  router.get("/health", (_, response) => {
    logger.info("PONG!");
    response.json({ status: "ok" });
  });
  router.use(backendCommon.errorHandler());
  return router;
}

const awsCodebuildPlugin = backendPluginApi.createBackendPlugin({
  pluginId: "aws-codebuild",
  register(env) {
    env.registerInit({
      deps: {
        logger: backendPluginApi.coreServices.logger,
        httpRouter: backendPluginApi.coreServices.httpRouter,
        config: backendPluginApi.coreServices.rootConfig,
        catalogApi: alpha.catalogServiceRef
      },
      async init({ logger, httpRouter, config, catalogApi }) {
        const winstonLogger = backendCommon.loggerToWinstonLogger(logger);
        const awsCodeBuildApi = await DefaultAwsCodeBuildService.fromConfig(
          config,
          {
            catalogApi,
            logger: winstonLogger
          }
        );
        httpRouter.use(
          await createRouter({
            logger: winstonLogger,
            awsCodeBuildApi
          })
        );
      }
    });
  }
});

exports.DefaultAwsCodeBuildService = DefaultAwsCodeBuildService;
exports.createRouter = createRouter;
exports["default"] = awsCodebuildPlugin;
//# sourceMappingURL=index.cjs.js.map
